@@ -7,7 +7,7 @@
 #  http://xml.coverpages.org/mpeg21-didl.html
 
 from typing import Any, Dict  # noqa: F401 pylint: disable=unused-import
-from typing import cast, List, Optional, Sequence, Type, TypeVar, Union
+from typing import List, Optional, Sequence, Type, TypeVar, Union
 from xml.etree import ElementTree as ET
 
 import defusedxml.ElementTree  # type: ignore
@@ -25,9 +25,13 @@ TD = TypeVar('TD', bound='Descriptor')
 TR = TypeVar('TR', bound='Resource')
 
 
+class DidlLiteException(Exception):
+    """DIDL Lite Exception."""
+
+
 # region: DidlObjects
 class DidlObject:
-    """DIDL Ojbect."""
+    """DIDL Object."""
 
     tag = None  # type: Optional[str]
     upnp_class = 'object'
@@ -42,25 +46,32 @@ class DidlObject:
         ('upnp', 'writeStatus', 'O'),
     ]
 
-    def __init__(self, id: str = "", parent_id: str = "",
-                 descriptors: Optional[Sequence['Descriptor']] = None, **properties: Any):
-        """Initializer."""
-        # pylint: disable=invalid-name,redefined-builtin
+    def __init__(
+        self, id: str = "", parent_id: str = "",
+        descriptors: Optional[Sequence['Descriptor']] = None,
+        xml_el: Optional[ET.Element] = None, strict: bool = True,
+        **properties: Any
+    ) -> None:
+        """Initialize."""
+        # pylint: disable=invalid-name,redefined-builtin,too-many-arguments
         properties['id'] = id
         properties['parent_id'] = parent_id
         properties['class'] = self.upnp_class
-        self._ensure_required_properties(**properties)
+        self._ensure_required_properties(strict, **properties)
         self._set_properties(**properties)
 
+        self.xml_el = xml_el
         self.resources = properties.get('resources') or []
         self.descriptors = descriptors if descriptors else []
 
-    def _ensure_required_properties(self, **properties: Any) -> None:
+    def _ensure_required_properties(self, strict: bool = True, **properties: Any) -> None:
         """Check if all required properties are given."""
         for property_def in self.didl_properties_defs:
             key = didl_property_def_key(property_def)
-            if property_def[2] == 'R' and key not in properties:
-                raise Exception(key + ' is mandatory')
+            if strict and \
+               property_def[2] == 'R' and \
+               key not in properties:
+                raise DidlLiteException(key + ' is mandatory')
 
     def _set_properties(self, **properties: Any) -> None:
         """Set attributes from properties."""
@@ -73,7 +84,7 @@ class DidlObject:
             setattr(self, key, value)
 
     @classmethod
-    def from_xml(cls: Type[TDO], xml_el: ET.Element) -> TDO:
+    def from_xml(cls: Type[TDO], xml_el: ET.Element, strict: bool = True) -> TDO:
         """
         Initialize from an XML node.
 
@@ -116,7 +127,7 @@ class DidlObject:
             descriptor = Descriptor.from_xml(desc_el)
             descriptors.append(descriptor)
 
-        return cls(descriptors=descriptors, **properties)
+        return cls(xml_el=xml_el, descriptors=descriptors, strict=strict, **properties)
 
     def to_xml(self) -> ET.Element:
         """Convert self to XML Element."""
@@ -525,16 +536,16 @@ class Container(DidlObject, list):
     ]
 
     @classmethod
-    def from_xml(cls: Type[TC], xml_el: ET.Element) -> TC:
+    def from_xml(cls: Type[TC], xml_el: ET.Element, strict: bool = True) -> TC:
         """
         Initialize from an XML node.
 
         I.e., parse XML and return instance.
         """
-        instance = super().from_xml(xml_el)
+        instance = super().from_xml(xml_el, strict)
 
         # add all children
-        didl_objects = from_xml_el(xml_el)
+        didl_objects = from_xml_el(xml_el, strict)
         instance.extend(didl_objects)  # pylint: disable=no-member
 
         return instance
@@ -797,13 +808,15 @@ class Resource:
 
     # pylint: disable=too-few-public-methods,too-many-instance-attributes
 
-    def __init__(self, uri: Optional[str], protocol_info: str, import_uri: Optional[str] = None,
-                 size: Optional[str] = None, duration: Optional[str] = None,
-                 bitrate: Optional[str] = None, sample_frequency: Optional[str] = None,
-                 bits_per_sample: Optional[str] = None, nr_audio_channels: Optional[str] = None,
-                 resolution: Optional[str] = None, color_depth: Optional[str] = None,
-                 protection: Optional[str] = None):
-        """Initializer."""
+    def __init__(
+        self, uri: Optional[str], protocol_info: Optional[str], import_uri: Optional[str] = None,
+        size: Optional[str] = None, duration: Optional[str] = None,
+        bitrate: Optional[str] = None, sample_frequency: Optional[str] = None,
+        bits_per_sample: Optional[str] = None, nr_audio_channels: Optional[str] = None,
+        resolution: Optional[str] = None, color_depth: Optional[str] = None,
+        protection: Optional[str] = None, xml_el: Optional[ET.Element] = None,
+    ) -> None:
+        """Initialize."""
         # pylint: disable=too-many-arguments
         self.uri = uri
         self.protocol_info = protocol_info
@@ -817,34 +830,35 @@ class Resource:
         self.resolution = resolution
         self.color_depth = color_depth
         self.protection = protection
+        self.xml_el = xml_el
 
     @classmethod
-    def from_xml(cls: Type[TR], xml_node: ET.Element) -> TR:
+    def from_xml(cls: Type[TR], xml_el: ET.Element) -> TR:
         """Initialize from an XML node."""
-        uri = xml_node.text
-        protocol_info = xml_node.attrib["protocolInfo"]
-        import_uri = xml_node.attrib.get('importUri')
-        size = xml_node.attrib.get('size')
-        duration = xml_node.attrib.get('duration')
-        bitrate = xml_node.attrib.get('bitrate')
-        sample_frequency = xml_node.attrib.get('sampleFrequency')
-        bits_per_sample = xml_node.attrib.get('bitsPerSample')
-        nr_audio_channels = xml_node.attrib.get('nrAudioChannels')
-        resolution = xml_node.attrib.get('resolution')
-        color_depth = xml_node.attrib.get('colorDepth')
-        protection = xml_node.attrib.get('protection')
-        return cls(uri, protocol_info,
+        uri = xml_el.text
+        protocol_info = xml_el.attrib.get("protocolInfo")
+        import_uri = xml_el.attrib.get('importUri')
+        size = xml_el.attrib.get('size')
+        duration = xml_el.attrib.get('duration')
+        bitrate = xml_el.attrib.get('bitrate')
+        sample_frequency = xml_el.attrib.get('sampleFrequency')
+        bits_per_sample = xml_el.attrib.get('bitsPerSample')
+        nr_audio_channels = xml_el.attrib.get('nrAudioChannels')
+        resolution = xml_el.attrib.get('resolution')
+        color_depth = xml_el.attrib.get('colorDepth')
+        protection = xml_el.attrib.get('protection')
+        return cls(uri, protocol_info=protocol_info,
                    import_uri=import_uri, size=size, duration=duration,
                    bitrate=bitrate, sample_frequency=sample_frequency,
                    bits_per_sample=bits_per_sample,
                    nr_audio_channels=nr_audio_channels,
                    resolution=resolution, color_depth=color_depth,
-                   protection=protection)
+                   protection=protection, xml_el=xml_el,)
 
     def to_xml(self) -> ET.Element:
         """Convert self to XML."""
         attribs = {
-            'protocolInfo': self.protocol_info,
+            'protocolInfo': self.protocol_info or '',
         }
         res_el = ET.Element('res', attribs)
         res_el.text = self.uri
@@ -854,22 +868,26 @@ class Resource:
 class Descriptor:
     """DIDL Descriptor."""
 
-    def __init__(self, id: str, name_space: str, type: Optional[str] = None,
-                 text: Optional[str] = None):
-        """Initializer."""
-        # pylint: disable=invalid-name,redefined-builtin
+    def __init__(
+        self, id: str, name_space: str, type: Optional[str] = None,
+        text: Optional[str] = None, xml_el: Optional[ET.Element] = None,
+    ) -> None:
+        """Initialize."""
+        # pylint: disable=invalid-name,redefined-builtin,too-many-arguments
         self.id = id
         self.name_space = name_space
         self.type = type
         self.text = text
+        self.xml_el = xml_el
 
     @classmethod
-    def from_xml(cls: Type[TD], xml_node: ET.Element) -> TD:
+    def from_xml(cls: Type[TD], xml_el: ET.Element) -> TD:
         """Initialize from an XML node."""
-        id_ = xml_node.attrib['id']
-        name_space = xml_node.attrib['nameSpace']
-        type_ = xml_node.attrib.get('type')
-        return cls(id_, name_space, type_, xml_node.text)
+        id_ = xml_el.attrib['id']
+        name_space = xml_el.attrib['nameSpace']
+        type_ = xml_el.attrib.get('type')
+        text = xml_el.text
+        return cls(id_, name_space, type=type_, text=text, xml_el=xml_el)
 
     def to_xml(self) -> ET.Element:
         """Convert self to XML."""
@@ -903,16 +921,16 @@ def to_xml_string(*objects: DidlObject) -> bytes:
         didl_object_el = didl_object.to_xml()
         root_el.append(didl_object_el)
 
-    return cast(bytes, ET.tostring(root_el))
+    return ET.tostring(root_el)
 
 
-def from_xml_string(xml_string: str) -> List[Union[DidlObject, Descriptor]]:
+def from_xml_string(xml_string: str, strict: bool = True) -> List[Union[DidlObject, Descriptor]]:
     """Convert XML string to DIDL Objects."""
     xml_el = defusedxml.ElementTree.fromstring(xml_string)
-    return from_xml_el(xml_el)
+    return from_xml_el(xml_el, strict)
 
 
-def from_xml_el(xml_el: ET.Element) -> List[Union[DidlObject, Descriptor]]:
+def from_xml_el(xml_el: ET.Element, strict: bool = True) -> List[Union[DidlObject, Descriptor]]:
     """Convert XML Element to DIDL Objects."""
     didl_objects = []  # type: List[Union[DidlObject, Descriptor]]
 
@@ -929,7 +947,7 @@ def from_xml_el(xml_el: ET.Element) -> List[Union[DidlObject, Descriptor]]:
         didl_object_type = type_by_upnp_class(upnp_class.text)
         if didl_object_type is None:
             continue
-        didl_object = didl_object_type.from_xml(child_el)
+        didl_object = didl_object_type.from_xml(child_el, strict)
         didl_objects.append(didl_object)
 
     # descriptors
